
/* Chrysalide - Outil d'analyse de fichiers binaires
 * thread.c - interactions avec les différents threads
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "thread.h"


#include <malloc.h>
#include <string.h>


#include "../misc/id.h"
#include "../misc/location.h"
#include "../misc/types.h"
#include "../../../common/endianness.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : req   = structure de réponse à constituer.                   *
*                sizes = références pour la valeur des tailles dynamiques.    *
*                blob  = ensemble de données binaires brutes. [OUT]           *
*                len   = quantité de données disponibles, puis écrites. [OUT] *
*                                                                             *
*  Description : Prépare une requête demandant le nom d'un thread.            *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool set_jdwp_thread_name(const jdwp_cmd_thread_name_request *req, const jdwp_cmd_vm_id_sizes_reply *sizes, bin_t *blob, off_t *len)
{
    bool result;                            /* Bilan à retourner           */
    off_t pos;                              /* Tête de lecture             */

    pos = 0;
    memset(blob, 0, sizeof(jdwp_cmd_thread_name_request));

    result = set_jdwp_thread_id(&req->id, sizes, blob, &pos, *len);
    if (!result) return false;

    *len = pos;

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : blob  = ensemble de données binaires brutes.                 *
*                len   = quantité de données valides.                         *
*                sizes = références pour la valeur des tailles dynamiques.    *
*                reply = structure de réponse à constituer. [OUT]             *
*                                                                             *
*  Description : Reconstitue une réponse fournissant le nom d'un thread.      *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool get_jdwp_thread_name(const bin_t *blob, off_t len, const jdwp_cmd_vm_id_sizes_reply *sizes, jdwp_cmd_thread_name_reply *reply)
{
    bool result;                            /* Bilan à retourner           */
    off_t pos;                              /* Tête de lecture             */

    pos = 0;
    memset(reply, 0, sizeof(jdwp_cmd_thread_name_reply));

    result = get_jdwp_string(blob, &pos, len, &reply->name);
    if (!result) return false;

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reply = structure de réponse à supprimer de la mémoire.      *
*                                                                             *
*  Description : Libère le nom donné à un thread.                             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void free_jdwp_thread_name_reply(jdwp_cmd_thread_name_reply *reply)
{
    free_jdwp_string(&reply->name);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : req   = structure de réponse à constituer.                   *
*                sizes = références pour la valeur des tailles dynamiques.    *
*                blob  = ensemble de données binaires brutes. [OUT]           *
*                len   = quantité de données disponibles, puis écrites. [OUT] *
*                                                                             *
*  Description : Prépare une requête demandant les frames d'un thread.        *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool set_jdwp_thread_frames(const jdwp_cmd_thread_frames_request *req, const jdwp_cmd_vm_id_sizes_reply *sizes, bin_t *blob, off_t *len)
{
    bool result;                            /* Bilan à retourner           */
    off_t pos;                              /* Tête de lecture             */

    pos = 0;
    memset(blob, 0, sizeof(jdwp_cmd_thread_frames_request));

    result = set_jdwp_frame_id(&req->id, sizes, blob, &pos, *len);
    if (!result) return false;

    result = write_u32(&req->start, blob, &pos, *len, SRE_BIG);
    if (!result) return false;

    result = write_u32(&req->length, blob, &pos, *len, SRE_BIG);
    if (!result) return false;

    *len = pos;

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : blob  = ensemble de données binaires brutes.                 *
*                len   = quantité de données valides.                         *
*                sizes = références pour la valeur des tailles dynamiques.    *
*                reply = structure de réponse à constituer. [OUT]             *
*                                                                             *
*  Description : Reconstitue une réponse fournissant les frames d'un thread.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool get_jdwp_thread_frames(const bin_t *blob, off_t len, const jdwp_cmd_vm_id_sizes_reply *sizes, jdwp_cmd_thread_frames_reply *reply)
{
    bool result;                            /* Bilan à retourner           */
    off_t pos;                              /* Tête de lecture             */
    uint32_t i;                             /* Boucle de parcours          */

    pos = 0;
    memset(reply, 0, sizeof(jdwp_cmd_thread_frames_reply));

    result = read_u32(&reply->count, blob, &pos, len, SRE_BIG);
    if (!result) return false;

    reply->frames = (jdwp_thread_frame *)calloc(reply->count, sizeof(jdwp_thread_frame));

    for (i = 0; i < reply->count && result; i++)
    {
        result = get_jdwp_frame_id(blob, &pos, len, sizes, &reply->frames[i].frame_id);

        result &= get_jdwp_location(blob, &pos, len, sizes, &reply->frames[i].location);

    }

    if (!result)
        free_jdwp_thread_frames_reply(reply);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reply = structure de réponse à supprimer de la mémoire.      *
*                                                                             *
*  Description : Libère une liste de frames d'un thread.                      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void free_jdwp_thread_frames_reply(jdwp_cmd_thread_frames_reply *reply)
{
    if (reply->frames != NULL)
        free(reply->frames);

}
